/// <reference types="graphql" />
import { GraphQLInterfaceType, GraphQLNonNull } from 'graphql';
import { StrongGraphQLOutputType } from './type';
import { StrongGraphQLObjectType, StrongGraphQLFieldConfig, StrongGraphQLFieldConfigWithArgs } from './object';
/**
 * Creates the basis for a strong GraphQL interface type. To add fields just
 * call `field` and `fieldNonNull`.
 *
 * The type when used will be non-null. In order to get the nullable variant of
 * the type just call `.nullable()`.
 */
export declare function createInterfaceType<TValue>(config: StrongGraphQLInterfaceTypeConfig<TValue>): StrongGraphQLInterfaceType<TValue>;
/**
 * The configuration object type for creating interface types.
 */
export declare type StrongGraphQLInterfaceTypeConfig<TValue> = {
    readonly name: string;
    readonly description?: string | undefined;
    readonly resolveType?: (value: TValue) => StrongGraphQLObjectType<TValue, any>;
};
/**
 * The type returned by `createInterfaceType`. It is non-null, to get the
 * nullable variant just call `.nullable()`.
 */
export declare class StrongGraphQLInterfaceType<TValue> extends GraphQLNonNull<StrongGraphQLNullableInterfaceType<TValue>> implements StrongGraphQLOutputType<TValue> {
    readonly _strongType: true;
    readonly _strongOutputType: true;
    readonly _strongValue: TValue | null;
    /**
     * The name of our interface type.
     */
    readonly name: string;
    constructor(nullableType: StrongGraphQLNullableInterfaceType<TValue>);
    getWeakType(): this;
    getWeakOutputType(): this;
    /**
     * Returns a new strong GraphQL object type with a new field. This function
     * does not mutate the type it was called on.
     *
     * The field created will have a nullable type. To get a non-null field type
     * use `fieldNonNull`.
     */
    field<TFieldValue>(config: StrongGraphQLFieldConfig<TValue, {}, never, TFieldValue | null | undefined>): StrongGraphQLInterfaceType<TValue>;
    field<TFieldValue, TArgs>(config: StrongGraphQLFieldConfigWithArgs<TValue, TArgs, never, TFieldValue | null | undefined>): StrongGraphQLInterfaceType<TValue>;
    /**
     * Returns a new strong GraphQL object type with a new field. This function
     * does not mutate the type it was called on.
     */
    fieldNonNull<TFieldValue>(config: StrongGraphQLFieldConfig<TValue, {}, never, TFieldValue>): StrongGraphQLInterfaceType<TValue>;
    fieldNonNull<TFieldValue, TArgs>(config: StrongGraphQLFieldConfigWithArgs<TValue, TArgs, never, TFieldValue>): StrongGraphQLInterfaceType<TValue>;
    /**
     * Returns the inner nullable version of this type without mutating anything.
     */
    nullable(): StrongGraphQLOutputType<TValue | null | undefined>;
}
/**
 * The nullable variant of the interface type that gets wrapped by the public
 * non-null variant. Because of the way strong GraphQL works this is the object
 * that actually extends `GraphQLInterfaceType`.
 */
export declare class StrongGraphQLNullableInterfaceType<TValue> extends GraphQLInterfaceType implements StrongGraphQLOutputType<TValue> {
    readonly _strongType: true;
    readonly _strongOutputType: true;
    readonly _strongValue: TValue | null;
    private readonly _strongConfig;
    private readonly _strongFieldConfigs;
    constructor(config: StrongGraphQLInterfaceTypeConfig<TValue>, fieldConfigs: Array<StrongGraphQLFieldConfig<TValue, {}, never, any>>);
    getWeakType(): this;
    getWeakOutputType(): this;
    /**
     * Returns true if we already have a field of this name.
     */
    private _hasField(fieldName);
    /**
     * Throws an error if we already have a field with the provided name,
     * otherwise the function does nothing.
     */
    private _assertUniqueFieldName(fieldName);
    /**
     * This field is a private implementation detail and should not be used
     * outside of `StrongGraphQLObjectType`!
     */
    _field<TFieldValue, TArgs>(config: StrongGraphQLFieldConfig<TValue, TArgs, never, TFieldValue | null | undefined>): StrongGraphQLNullableInterfaceType<TValue>;
    /**
     * This field is a private implementation detail and should not be used
     * outside of `StrongGraphQLObjectType`!
     */
    _fieldNonNull<TFieldValue, TArgs>(config: StrongGraphQLFieldConfig<TValue, TArgs, never, TFieldValue>): StrongGraphQLNullableInterfaceType<TValue>;
    /**
     * Returns self.
     */
    nullable(): this;
}
