"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
const graphql_1 = require("graphql");
const object_1 = require("./object");
const description_1 = require("./description");
/**
 * Creates the basis for a strong GraphQL interface type. To add fields just
 * call `field` and `fieldNonNull`.
 *
 * The type when used will be non-null. In order to get the nullable variant of
 * the type just call `.nullable()`.
 */
function createInterfaceType(config) {
    return new StrongGraphQLInterfaceType(new StrongGraphQLNullableInterfaceType(description_1.trimDescriptionsInConfig(config), []));
}
exports.createInterfaceType = createInterfaceType;
/**
 * The type returned by `createInterfaceType`. It is non-null, to get the
 * nullable variant just call `.nullable()`.
 */
class StrongGraphQLInterfaceType extends graphql_1.GraphQLNonNull {
    constructor(nullableType) {
        super(nullableType);
        this.name = nullableType.name;
    }
    // The required type conversion methods.
    getWeakType() { return this; }
    getWeakOutputType() { return this; }
    field(config) {
        return new StrongGraphQLInterfaceType(this.ofType._field(config));
    }
    fieldNonNull(config) {
        return new StrongGraphQLInterfaceType(this.ofType._fieldNonNull(config));
    }
    /**
     * Returns the inner nullable version of this type without mutating anything.
     */
    nullable() {
        return this.ofType;
    }
}
exports.StrongGraphQLInterfaceType = StrongGraphQLInterfaceType;
/**
 * The nullable variant of the interface type that gets wrapped by the public
 * non-null variant. Because of the way strong GraphQL works this is the object
 * that actually extends `GraphQLInterfaceType`.
 */
class StrongGraphQLNullableInterfaceType extends graphql_1.GraphQLInterfaceType {
    constructor(config, fieldConfigs) {
        const { resolveType } = config;
        super({
            name: config.name,
            description: config.description,
            resolveType: resolveType && (value => resolveType(value).ofType),
            fields: () => object_1.createWeakFieldMap(fieldConfigs),
        });
        this._strongConfig = config;
        this._strongFieldConfigs = fieldConfigs;
    }
    // The required type conversion methods.
    getWeakType() { return this; }
    getWeakOutputType() { return this; }
    /**
     * Returns true if we already have a field of this name.
     */
    _hasField(fieldName) {
        return Boolean(this._strongFieldConfigs.find(({ name }) => name === fieldName));
    }
    /**
     * Throws an error if we already have a field with the provided name,
     * otherwise the function does nothing.
     */
    _assertUniqueFieldName(fieldName) {
        if (this._hasField(fieldName))
            throw new Error(`Type '${this.name}' already has a field named '${fieldName}'.`);
    }
    /**
     * This field is a private implementation detail and should not be used
     * outside of `StrongGraphQLObjectType`!
     */
    _field(config) {
        this._assertUniqueFieldName(config.name);
        if (config.resolve)
            throw new Error(`Field '${config.resolve}' has a resolver, but it will never be used.`);
        return new StrongGraphQLNullableInterfaceType(this._strongConfig, [...this._strongFieldConfigs, description_1.trimDescriptionsInConfig(__assign({}, config, { type: () => typeof config.type === 'function' ? config.type().nullable() : config.type.nullable() }))]);
    }
    /**
     * This field is a private implementation detail and should not be used
     * outside of `StrongGraphQLObjectType`!
     */
    _fieldNonNull(config) {
        this._assertUniqueFieldName(config.name);
        if (config.resolve)
            throw new Error(`Field '${config.resolve}' has a resolver, but it will never be used.`);
        return new StrongGraphQLNullableInterfaceType(this._strongConfig, [...this._strongFieldConfigs, description_1.trimDescriptionsInConfig(config)]);
    }
    /**
     * Returns self.
     */
    nullable() {
        return this;
    }
}
exports.StrongGraphQLNullableInterfaceType = StrongGraphQLNullableInterfaceType;
