/// <reference types="graphql" />
import { GraphQLNonNull, GraphQLObjectType, GraphQLFieldConfigMap, ExecutionResult } from 'graphql';
import { StrongGraphQLOutputType, StrongGraphQLInputType } from './type';
import { StrongGraphQLInterfaceType } from './interface';
/**
 * Creates a strong GraphQL object type with a fluent builder interface.
 *
 * The type will be non-null, in order to get the nullable variant of the type
 * just call `.nullable()`.
 */
export declare function createObjectType<TValue>(config: StrongGraphQLObjectTypeConfig<TValue, {}>): StrongGraphQLObjectType<TValue, {}>;
export declare function createObjectType<TValue, TContext>(config: StrongGraphQLObjectTypeConfig<TValue, TContext>): StrongGraphQLObjectType<TValue, TContext>;
/**
 * A configuration object to be used when creating object types.
 */
export declare type StrongGraphQLObjectTypeConfig<TValue, TContext> = {
    readonly name: string;
    readonly description?: string | undefined;
    readonly isTypeOf?: (value: any, context: TContext) => value is TValue;
    readonly interfaces?: Array<StrongGraphQLInterfaceType<TValue>> | (() => Array<StrongGraphQLInterfaceType<TValue>>);
};
/**
 * The object returned by `createObjectType`. It is non-null, to get the
 * nullable variant just call `.nullable()`.
 */
export declare class StrongGraphQLObjectType<TValue, TContext> extends GraphQLNonNull<StrongGraphQLNullableObjectType<TValue, TContext>> implements StrongGraphQLOutputType<TValue> {
    readonly _strongType: true;
    readonly _strongOutputType: true;
    readonly _strongValue: TValue | null;
    /**
     * A schema created for executing queries against where the query type is this
     * object type.
     */
    private _schema;
    /**
     * The name of our object type.
     */
    readonly name: string;
    constructor(nullableType: StrongGraphQLNullableObjectType<TValue, TContext>);
    getWeakType(): this;
    getWeakOutputType(): this;
    /**
     * Returns a new strong GraphQL object type with a new field. This function
     * does not mutate the type it was called on.
     *
     * The field created will have a nullable type. To get a non-null field type
     * use `fieldNonNull`.
     */
    field<TFieldValue>(config: StrongGraphQLFieldConfig<TValue, {}, TContext, TFieldValue | null | undefined>): StrongGraphQLObjectType<TValue, TContext>;
    field<TFieldValue, TArgs>(config: StrongGraphQLFieldConfigWithArgs<TValue, TArgs, TContext, TFieldValue | null | undefined>): StrongGraphQLObjectType<TValue, TContext>;
    /**
     * Returns a new strong GraphQL object type with a new field. This function
     * does not mutate the type it was called on.
     */
    fieldNonNull<TFieldValue>(config: StrongGraphQLFieldConfig<TValue, {}, TContext, TFieldValue>): StrongGraphQLObjectType<TValue, TContext>;
    fieldNonNull<TFieldValue, TArgs>(config: StrongGraphQLFieldConfigWithArgs<TValue, TArgs, TContext, TFieldValue>): StrongGraphQLObjectType<TValue, TContext>;
    /**
     * Extends the object type be calling a function which takes the object as an
     * input and returns an object of the same type. This allows the creation of
     * simple extensions that leverage the immutable builder pattern used by this
     * library.
     */
    extend(extension: (type: this) => this): this;
    /**
     * Returns the inner nullable version of this type without mutating anything.
     */
    nullable(): StrongGraphQLOutputType<TValue | null | undefined>;
    /**
     * Creates a new copy of this type. It is the exact same as the type which
     * `.clone()` was called on except that the reference is different.
     */
    clone(): StrongGraphQLObjectType<TValue, TContext>;
    /**
     * Executes a GraphQL query against this type. The schema used for executing
     * this query uses this object type as the query object type. There is no
     * mutation or subscription type.
     *
     * This can be very useful in testing.
     */
    execute(query: string, value: TValue, context: TContext, variables?: {
        [key: string]: any;
    }, operation?: string): Promise<ExecutionResult>;
}
/**
 * The private nullable implementation of `StrongGraphQLObjectType`. Because we
 * want types to be non-null by default, but in GraphQL types are nullable by
 * default this type is also the one that actually extends from
 * `GraphQLObjectType`.
 */
export declare class StrongGraphQLNullableObjectType<TValue, TContext> extends GraphQLObjectType implements StrongGraphQLOutputType<TValue | null | undefined> {
    readonly _strongType: true;
    readonly _strongOutputType: true;
    readonly _strongValue: TValue | null;
    private readonly _strongConfig;
    private readonly _strongFieldConfigs;
    constructor(config: StrongGraphQLObjectTypeConfig<TValue, TContext>, fieldConfigs: Array<StrongGraphQLFieldConfig<TValue, {}, TContext, any>>);
    getWeakType(): this;
    getWeakOutputType(): this;
    /**
     * Returns true if we already have a field of this name.
     */
    private _hasField(fieldName);
    /**
     * Throws an error if we already have a field with the provided name,
     * otherwise the function does nothing.
     */
    private _assertUniqueFieldName(fieldName);
    /**
     * This field is a private implementation detail and should not be used
     * outside of `StrongGraphQLObjectType`!
     */
    _field<TFieldValue, TArgs>(config: StrongGraphQLFieldConfig<TValue, TArgs, TContext, TFieldValue | null | undefined>): StrongGraphQLNullableObjectType<TValue, TContext>;
    /**
     * This field is a private implementation detail and should not be used
     * outside of `StrongGraphQLObjectType`!
     */
    _fieldNonNull<TFieldValue, TArgs>(config: StrongGraphQLFieldConfig<TValue, TArgs, TContext, TFieldValue>): StrongGraphQLNullableObjectType<TValue, TContext>;
    /**
     * Returns self.
     */
    nullable(): this;
    /**
     * Creates a new copy of this type. It is the exact same as the type which
     * `.clone()` was called on except that the reference is different.
     */
    clone(): StrongGraphQLNullableObjectType<TValue, TContext>;
}
/**
 * A type which represents the GraphQL type definition of the argument
 * TypeScript type provided.
 */
export declare type StrongGraphQLArgsConfig<TArgs> = {
    [TArg in keyof TArgs]: StrongGraphQLArgConfig<TArgs[TArg]>;
};
/**
 * A type which represents a single argument configuration.
 */
export declare type StrongGraphQLArgConfig<TValue> = {
    readonly type: StrongGraphQLInputType<TValue>;
    readonly defaultValue?: TValue;
    readonly description?: string | undefined;
};
/**
 * The configration object for a single field of a strong GraphQL object type.
 * Takes a lot of generic type parameters to make sure everything is super safe!
 *
 * Arguments are optional.
 */
export declare type StrongGraphQLFieldConfig<TSourceValue, TArgs, TContext, TValue> = {
    readonly name: string;
    readonly description?: string | undefined;
    readonly deprecationReason?: string | undefined;
    readonly type: StrongGraphQLOutputType<TValue> | (() => StrongGraphQLOutputType<TValue>);
    readonly args?: StrongGraphQLArgsConfig<TArgs>;
    readonly resolve?: (source: TSourceValue, args: TArgs, context: TContext) => TValue | Promise<TValue>;
};
/**
 * A single field configuration except the arguments are required.
 */
export declare type StrongGraphQLFieldConfigWithArgs<TSourceValue, TArgs, TContext, TValue> = StrongGraphQLFieldConfig<TSourceValue, TArgs, TContext, TValue> & {
    readonly args: StrongGraphQLArgsConfig<TArgs>;
};
/**
 * Creates a weak field map using an array of strong GraphQL field configs. This
 * is generally called within a field thunk.
 *
 * This is private API and not exposed to users.
 */
export declare function createWeakFieldMap<TValue, TContext>(fieldConfigs: Array<StrongGraphQLFieldConfig<TValue, {}, TContext, any>>): GraphQLFieldConfigMap<TValue, TContext>;
