"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
const graphql_1 = require("graphql");
const description_1 = require("./description");
function createObjectType(config) {
    return new StrongGraphQLObjectType(new StrongGraphQLNullableObjectType(description_1.trimDescriptionsInConfig(config), []));
}
exports.createObjectType = createObjectType;
/**
 * The object returned by `createObjectType`. It is non-null, to get the
 * nullable variant just call `.nullable()`.
 */
// Developers could just instantiate this object directly instead of using
// `createObjectType`, but the function interface feels nicer and allows us to
// add extra features like function overloading.
class StrongGraphQLObjectType extends graphql_1.GraphQLNonNull {
    constructor(nullableType) {
        super(nullableType);
        this.name = nullableType.name;
    }
    // The required type conversion methods.
    getWeakType() { return this; }
    getWeakOutputType() { return this; }
    field(config) {
        return new StrongGraphQLObjectType(this.ofType._field(config));
    }
    fieldNonNull(config) {
        return new StrongGraphQLObjectType(this.ofType._fieldNonNull(config));
    }
    /**
     * Extends the object type be calling a function which takes the object as an
     * input and returns an object of the same type. This allows the creation of
     * simple extensions that leverage the immutable builder pattern used by this
     * library.
     */
    extend(extension) {
        return extension(this);
    }
    /**
     * Returns the inner nullable version of this type without mutating anything.
     */
    nullable() {
        return this.ofType;
    }
    /**
     * Creates a new copy of this type. It is the exact same as the type which
     * `.clone()` was called on except that the reference is different.
     */
    clone() {
        return new StrongGraphQLObjectType(this.ofType.clone());
    }
    /**
     * Executes a GraphQL query against this type. The schema used for executing
     * this query uses this object type as the query object type. There is no
     * mutation or subscription type.
     *
     * This can be very useful in testing.
     */
    execute(query, value, context, variables = {}, operation) {
        if (this._schema == null)
            this._schema = new graphql_1.GraphQLSchema({ query: this.ofType });
        return graphql_1.graphql(this._schema, query, value, context, variables, operation);
    }
}
exports.StrongGraphQLObjectType = StrongGraphQLObjectType;
/**
 * The private nullable implementation of `StrongGraphQLObjectType`. Because we
 * want types to be non-null by default, but in GraphQL types are nullable by
 * default this type is also the one that actually extends from
 * `GraphQLObjectType`.
 */
class StrongGraphQLNullableObjectType extends graphql_1.GraphQLObjectType {
    constructor(config, fieldConfigs) {
        const { interfaces } = config;
        super({
            name: config.name,
            description: config.description,
            isTypeOf: config.isTypeOf,
            interfaces: interfaces && (() => (typeof interfaces === 'function' ? interfaces() : interfaces).map(({ ofType }) => ofType)),
            // We define a thunk which computes our fields from the fields config
            // array we’ve built.
            fields: () => createWeakFieldMap(fieldConfigs),
        });
        this._strongConfig = config;
        this._strongFieldConfigs = fieldConfigs;
    }
    // The required type conversion methods.
    getWeakType() { return this; }
    getWeakOutputType() { return this; }
    /**
     * Returns true if we already have a field of this name.
     */
    _hasField(fieldName) {
        return Boolean(this._strongFieldConfigs.find(({ name }) => name === fieldName));
    }
    /**
     * Throws an error if we already have a field with the provided name,
     * otherwise the function does nothing.
     */
    _assertUniqueFieldName(fieldName) {
        if (this._hasField(fieldName))
            throw new Error(`Type '${this.name}' already has a field named '${fieldName}'.`);
    }
    /**
     * This field is a private implementation detail and should not be used
     * outside of `StrongGraphQLObjectType`!
     */
    _field(config) {
        this._assertUniqueFieldName(config.name);
        return new StrongGraphQLNullableObjectType(this._strongConfig, [...this._strongFieldConfigs, description_1.trimDescriptionsInConfig(__assign({}, config, { type: () => typeof config.type === 'function' ? config.type().nullable() : config.type.nullable() }))]);
    }
    /**
     * This field is a private implementation detail and should not be used
     * outside of `StrongGraphQLObjectType`!
     */
    _fieldNonNull(config) {
        this._assertUniqueFieldName(config.name);
        return new StrongGraphQLNullableObjectType(this._strongConfig, [...this._strongFieldConfigs, description_1.trimDescriptionsInConfig(config)]);
    }
    /**
     * Returns self.
     */
    nullable() {
        return this;
    }
    /**
     * Creates a new copy of this type. It is the exact same as the type which
     * `.clone()` was called on except that the reference is different.
     */
    clone() {
        return new StrongGraphQLNullableObjectType(this._strongConfig, this._strongFieldConfigs);
    }
}
exports.StrongGraphQLNullableObjectType = StrongGraphQLNullableObjectType;
/**
 * Creates a weak field map using an array of strong GraphQL field configs. This
 * is generally called within a field thunk.
 *
 * This is private API and not exposed to users.
 */
function createWeakFieldMap(fieldConfigs) {
    const fields = {};
    for (const fieldConfig of fieldConfigs) {
        // Create an args object that we will give to our field config. This
        // arguments object will be mutated later and filled with argument
        // configs.
        const argsDefinition = {};
        fields[fieldConfig.name] = {
            description: fieldConfig.description,
            deprecationReason: fieldConfig.deprecationReason,
            type: typeof fieldConfig.type === 'function' ? fieldConfig.type().getWeakOutputType() : fieldConfig.type.getWeakOutputType(),
            args: argsDefinition,
            resolve: (source, args, context) => {
                if (!fieldConfig.resolve)
                    throw new Error(`Field '${fieldConfig.name}' requires a resolver.`);
                return fieldConfig.resolve(source, args, context);
            },
        };
        // If the field has defined some arguments, loop through the arguments
        // that exist and add them to the `args` object.
        if (fieldConfig.args) {
            for (const argName in fieldConfig.args) {
                if (fieldConfig.args.hasOwnProperty(argName)) {
                    const argConfig = fieldConfig.args[argName];
                    argsDefinition[argName] = {
                        type: argConfig.type.getWeakInputType(),
                        defaultValue: argConfig.defaultValue,
                        description: argConfig.description,
                    };
                }
            }
        }
    }
    return fields;
}
exports.createWeakFieldMap = createWeakFieldMap;
